@@ -74,7 +74,7 @@ static void applyMask(const uint8_t functionModules[], uint8_t qrcode[], enum qr
 static long getPenaltyScore(const uint8_t qrcode[]);
 static int finderPenaltyCountPatterns(const int runHistory[7], int qrsize);
 static int finderPenaltyTerminateAndCount(bool currentRunColor, int currentRunLength, int runHistory[7], int qrsize);
-static void finderPenaltyAddHistory(int currentRunLength, int runHistory[7]);
+static void finderPenaltyAddHistory(int currentRunLength, int runHistory[7], int qrsize);
 
 testable bool getModule(const uint8_t qrcode[], int x, int y);
 testable void setModule(uint8_t qrcode[], int x, int y, bool isBlack);
@@ -132,7 +132,7 @@ bool qrcodegen_encodeText(const char *text, uint8_t tempBuffer[], uint8_t qrcode
 	size_t textLen = strlen(text);
 	if (textLen == 0)
 		return qrcodegen_encodeSegmentsAdvanced(NULL, 0, ecl, minVersion, maxVersion, mask, boostEcl, tempBuffer, qrcode);
-	size_t bufLen = qrcodegen_BUFFER_LEN_FOR_VERSION(maxVersion);
+	size_t bufLen = (size_t)qrcodegen_BUFFER_LEN_FOR_VERSION(maxVersion);
 	
 	struct qrcodegen_Segment seg;
 	if (qrcodegen_isNumeric(text)) {
@@ -228,14 +228,16 @@ bool qrcodegen_encodeSegmentsAdvanced(const struct qrcodegen_Segment segs[], siz
 	}
 	
 	// Concatenate all segments to create the data bit string
-	memset(qrcode, 0, qrcodegen_BUFFER_LEN_FOR_VERSION(version) * sizeof(qrcode[0]));
+	memset(qrcode, 0, (size_t)qrcodegen_BUFFER_LEN_FOR_VERSION(version) * sizeof(qrcode[0]));
 	int bitLen = 0;
 	for (size_t i = 0; i < len; i++) {
 		const struct qrcodegen_Segment *seg = &segs[i];
-		appendBitsToBuffer((int)seg->mode, 4, qrcode, &bitLen);
-		appendBitsToBuffer(seg->numChars, numCharCountBits(seg->mode, version), qrcode, &bitLen);
-		for (int j = 0; j < seg->bitLength; j++)
-			appendBitsToBuffer((seg->data[j >> 3] >> (7 - (j & 7))) & 1, 1, qrcode, &bitLen);
+		appendBitsToBuffer((unsigned int)seg->mode, 4, qrcode, &bitLen);
+		appendBitsToBuffer((unsigned int)seg->numChars, numCharCountBits(seg->mode, version), qrcode, &bitLen);
+		for (int j = 0; j < seg->bitLength; j++) {
+			int bit = (seg->data[j >> 3] >> (7 - (j & 7))) & 1;
+			appendBitsToBuffer((unsigned int)bit, 1, qrcode, &bitLen);
+		}
 	}
 	assert(bitLen == dataUsedBits);
 	
@@ -343,6 +345,7 @@ testable int getNumRawDataModules(int ver) {
 		if (ver >= 7)
 			result -= 36;
 	}
+	assert(208 <= result && result <= 29648);
 	return result;
 }
 
@@ -356,7 +359,7 @@ testable void reedSolomonComputeDivisor(int degree, uint8_t result[]) {
 	assert(1 <= degree && degree <= qrcodegen_REED_SOLOMON_DEGREE_MAX);
 	// Polynomial coefficients are stored from highest to lowest power, excluding the leading term which is always 1.
 	// For example the polynomial x^3 + 255x^2 + 8x + 93 is stored as the uint8 array {255, 8, 93}.
-	memset(result, 0, degree * sizeof(result[0]));
+	memset(result, 0, (size_t)degree * sizeof(result[0]));
 	result[degree - 1] = 1;  // Start off with the monomial x^0
 	
 	// Compute the product polynomial (x - r^0) * (x - r^1) * (x - r^2) * ... * (x - r^{degree-1}),
@@ -381,10 +384,10 @@ testable void reedSolomonComputeDivisor(int degree, uint8_t result[]) {
 testable void reedSolomonComputeRemainder(const uint8_t data[], int dataLen,
 		const uint8_t generator[], int degree, uint8_t result[]) {
 	assert(1 <= degree && degree <= qrcodegen_REED_SOLOMON_DEGREE_MAX);
-	memset(result, 0, degree * sizeof(result[0]));
+	memset(result, 0, (size_t)degree * sizeof(result[0]));
 	for (int i = 0; i < dataLen; i++) {  // Polynomial division
 		uint8_t factor = data[i] ^ result[0];
-		memmove(&result[0], &result[1], (degree - 1) * sizeof(result[0]));
+		memmove(&result[0], &result[1], (size_t)(degree - 1) * sizeof(result[0]));
 		result[degree - 1] = 0;
 		for (int j = 0; j < degree; j++)
 			result[j] ^= reedSolomonMultiply(generator[j], factor);
@@ -400,7 +403,7 @@ testable uint8_t reedSolomonMultiply(uint8_t x, uint8_t y) {
 	// Russian peasant multiplication
 	uint8_t z = 0;
 	for (int i = 7; i >= 0; i--) {
-		z = (z << 1) ^ ((z >> 7) * 0x11D);
+		z = (uint8_t)((z << 1) ^ ((z >> 7) * 0x11D));
 		z ^= ((y >> i) & 1) * x;
 	}
 	return z;
@@ -415,7 +418,7 @@ testable uint8_t reedSolomonMultiply(uint8_t x, uint8_t y) {
 testable void initializeFunctionModules(int version, uint8_t qrcode[]) {
 	// Initialize QR Code
 	int qrsize = version * 4 + 17;
-	memset(qrcode, 0, ((qrsize * qrsize + 7) / 8 + 1) * sizeof(qrcode[0]));
+	memset(qrcode, 0, (size_t)((qrsize * qrsize + 7) / 8 + 1) * sizeof(qrcode[0]));
 	qrcode[0] = (uint8_t)qrsize;
 	
 	// Fill horizontal and vertical timing patterns
@@ -551,7 +554,7 @@ testable int getAlignmentPatternPositions(int version, uint8_t result[7]) {
 	int step = (version == 32) ? 26 :
 		(version*4 + numAlign*2 + 1) / (numAlign*2 - 2) * 2;
 	for (int i = numAlign - 1, pos = version * 4 + 10; i >= 1; i--, pos -= step)
-		result[i] = pos;
+		result[i] = (uint8_t)pos;
 	result[0] = 6;
 	return numAlign;
 }
@@ -639,7 +642,6 @@ static long getPenaltyScore(const uint8_t qrcode[]) {
 		bool runColor = false;
 		int runX = 0;
 		int runHistory[7] = {0};
-		int padRun = qrsize;  // Add white border to initial run
 		for (int x = 0; x < qrsize; x++) {
 			if (getModule(qrcode, x, y) == runColor) {
 				runX++;
@@ -648,22 +650,20 @@ static long getPenaltyScore(const uint8_t qrcode[]) {
 				else if (runX > 5)
 					result++;
 			} else {
-				finderPenaltyAddHistory(runX + padRun, runHistory);
-				padRun = 0;
+				finderPenaltyAddHistory(runX, runHistory, qrsize);
 				if (!runColor)
 					result += finderPenaltyCountPatterns(runHistory, qrsize) * PENALTY_N3;
 				runColor = getModule(qrcode, x, y);
 				runX = 1;
 			}
 		}
-		result += finderPenaltyTerminateAndCount(runColor, runX + padRun, runHistory, qrsize) * PENALTY_N3;
+		result += finderPenaltyTerminateAndCount(runColor, runX, runHistory, qrsize) * PENALTY_N3;
 	}
 	// Adjacent modules in column having same color, and finder-like patterns
 	for (int x = 0; x < qrsize; x++) {
 		bool runColor = false;
 		int runY = 0;
 		int runHistory[7] = {0};
-		int padRun = qrsize;  // Add white border to initial run
 		for (int y = 0; y < qrsize; y++) {
 			if (getModule(qrcode, x, y) == runColor) {
 				runY++;
@@ -672,15 +672,14 @@ static long getPenaltyScore(const uint8_t qrcode[]) {
 				else if (runY > 5)
 					result++;
 			} else {
-				finderPenaltyAddHistory(runY + padRun, runHistory);
-				padRun = 0;
+				finderPenaltyAddHistory(runY, runHistory, qrsize);
 				if (!runColor)
 					result += finderPenaltyCountPatterns(runHistory, qrsize) * PENALTY_N3;
 				runColor = getModule(qrcode, x, y);
 				runY = 1;
 			}
 		}
-		result += finderPenaltyTerminateAndCount(runColor, runY + padRun, runHistory, qrsize) * PENALTY_N3;
+		result += finderPenaltyTerminateAndCount(runColor, runY, runHistory, qrsize) * PENALTY_N3;
 	}
 	
 	// 2*2 blocks of modules having same color
@@ -726,17 +725,19 @@ static int finderPenaltyCountPatterns(const int runHistory[7], int qrsize) {
 // Must be called at the end of a line (row or column) of modules. A helper function for getPenaltyScore().
 static int finderPenaltyTerminateAndCount(bool currentRunColor, int currentRunLength, int runHistory[7], int qrsize) {
 	if (currentRunColor) {  // Terminate black run
-		finderPenaltyAddHistory(currentRunLength, runHistory);
+		finderPenaltyAddHistory(currentRunLength, runHistory, qrsize);
 		currentRunLength = 0;
 	}
 	currentRunLength += qrsize;  // Add white border to final run
-	finderPenaltyAddHistory(currentRunLength, runHistory);
+	finderPenaltyAddHistory(currentRunLength, runHistory, qrsize);
 	return finderPenaltyCountPatterns(runHistory, qrsize);
 }
 
 
 // Pushes the given value to the front and drops the last value. A helper function for getPenaltyScore().
-static void finderPenaltyAddHistory(int currentRunLength, int runHistory[7]) {
+static void finderPenaltyAddHistory(int currentRunLength, int runHistory[7], int qrsize) {
+	if (runHistory[0] == 0)
+		currentRunLength += qrsize;  // Add white border to initial run
 	memmove(&runHistory[1], &runHistory[0], 6 * sizeof(runHistory[0]));
 	runHistory[0] = currentRunLength;
 }
@@ -962,16 +963,16 @@ struct qrcodegen_Segment qrcodegen_makeEci(long assignVal, uint8_t buf[]) {
 		assert(false);
 	else if (assignVal < (1 << 7)) {
 		memset(buf, 0, 1 * sizeof(buf[0]));
-		appendBitsToBuffer(assignVal, 8, buf, &result.bitLength);
+		appendBitsToBuffer((unsigned int)assignVal, 8, buf, &result.bitLength);
 	} else if (assignVal < (1 << 14)) {
 		memset(buf, 0, 2 * sizeof(buf[0]));
 		appendBitsToBuffer(2, 2, buf, &result.bitLength);
-		appendBitsToBuffer(assignVal, 14, buf, &result.bitLength);
+		appendBitsToBuffer((unsigned int)assignVal, 14, buf, &result.bitLength);
 	} else if (assignVal < 1000000L) {
 		memset(buf, 0, 3 * sizeof(buf[0]));
 		appendBitsToBuffer(6, 3, buf, &result.bitLength);
-		appendBitsToBuffer(assignVal >> 10, 11, buf, &result.bitLength);
-		appendBitsToBuffer(assignVal & 0x3FF, 10, buf, &result.bitLength);
+		appendBitsToBuffer((unsigned int)(assignVal >> 10), 11, buf, &result.bitLength);
+		appendBitsToBuffer((unsigned int)(assignVal & 0x3FF), 10, buf, &result.bitLength);
 	} else
 		assert(false);
 	result.data = buf;
